<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/**
* Modular Extensions - HMVC
* 
* Adapted from the CodeIgniter Core Classes
* @copyright	Copyright (c) 2006, EllisLab, Inc.
* @license		http://codeigniter.com/user_guide/license.html
* 
* Description:
* This helper provides static functions to load and instantiate modules 
* from the modular_extensions library which can operate as HMVC controllers.
*  
* Modules have access to the currently loaded core classes in the same
* manner as a normal controller.
* 
* Install this file as application/helpers/modules_helper.php
* 
* Load manually or autoload as needed.
*
* @version: 4.0.26 (c) Wiredesignz 2008-03-25
**/

/* load the modular extensions library */
require_once APPPATH.'libraries/modular_extensions.php';

/* define the modules base path */
define('MODBASE', APPPATH.'modules/');

class Modules
{
	/** Load a module.
	 * 
	 * @return object reference [module]
	 **/	
	function & load($module, $base = 'controllers/') 
    {		
		is_array($module) ? list($module, $config) = each($module) : $config = '';
		
		list($path, $module, $home) = modules::path_to($module, $module, $base);
		
		$_module = strtolower($module);
		
		$ci =& get_instance();
		
		if (isset($ci->$_module))
		{
			return $ci->$_module;			
		}

		modules::load_file($module, $path);
		
		$ci->_me['home'] = $home;
		
		$ci->$_module =& new $module($config);
		
		$ci->load->_ci_assign_to_models();
		
		return $ci->$_module;
	}

	/** Load a file
	 * 
	 * @return array [$config] or [$language] 
	 * @return boolean [TRUE] for class
	 **/	
	function load_file($file, $path, $type = 'class', $result = TRUE)
	{ 
		$file = str_replace(EXT, '', $file);

		if ($type == 'class')
		{
			include_once $path.$file.EXT;
		}
		else
		{
			include $path.$file.EXT;
			
			if (!isset($$type) OR !is_array($$type))
			{		
				show_error($path.$file.EXT." does not contain a valid {$type} array");
			}
			
			$result = $$type;
		}
		
		log_message('debug', "File loaded: ".$path.$file.EXT);
		return $result;
	}
	
	/** Find a file
	 * 
	 * Scans for files located anywhere within application/modules directory.
	 * Also scans application directories for helpers, plugins and views.
	 * Generates fatal error on file not found.
	 * 
	 * @return array [$path, $file, $home]
	 * @return array [FALSE, $file] for some application files
	 **/	
	function path_to($file, $path = '', $base = 'controllers/')
	{
		$file .= strpos($file, '.') ? '' : EXT;
		
		if (($pos = strrpos($file, '/')) !== FALSE)
	    {
			$path  = substr($file, 0, $pos);
			$file  = substr($file, $pos + 1);
	    }
		
		$path .= '/';
		
		$path2file = $path.$base.$file;
		
		$paths2scan = array(MODBASE.$path.$base, MODBASE.$path, MODBASE.$base, MODBASE);
		
		/* don't scan application directories for modules, libraries or methods */
		if (!in_array($base, array('controllers/', 'libraries/', 'methods/')))
		{
			$paths2scan = array_merge($paths2scan, array(APPPATH.$base, APPPATH.$base.$path));
		}
		  
		foreach ($paths2scan as $path2)
		{				
			foreach (array($file, ucfirst($file)) as $name)
			{
				if (is_file($path2.$name))
				{
					return array($path2, str_replace(EXT, '', $name), str_replace('/','', $path));
				}
			}
		}
		
		/* module file not found */
		
		/* don't die just yet, we handle each of these types back in the loader */
		if (in_array($base, array('helpers/','libraries/', 'methods/', 'plugins/')))
		{
			return array(FALSE, str_replace(EXT, '', $file));
		}	
		
		/* ok, die now */
		show_error("Unable to locate the requested file: ".$path2file);
	}

	/** Run a module
	 * 
	 * @return mixed [module ouput]
	 **/
	function run($module, $data = '', $method = '')
    {
		$class =& modules::load($module);
		
		$method = ($method) ? $method : 'index';
		
		return method_exists($class, $method) ?  $class->$method($data) : $class->error($method, $data);
    }
	
	/** Debug
	 * 
	 * Lists object_vars
	 **/
	function debug($_this)
    {
        if (is_object($_this))
        {
            echo '<pre>',get_class($_this),' Object: '."\n",
            print_r(array_keys(get_object_vars($_this)), TRUE),'</pre>';
            echo '<br />';
        }
    }
 
	/** Debug_in
	 * 
	 * Dumps an object or array
	 **/
	function debug_in($_this)
    {
        if (is_object($_this))
        {
            echo '<pre>',get_class($_this),' Object: '."\n",
            print_r(get_object_vars($_this), TRUE),'</pre>';
            echo '<br />';
        }
		
		if (is_array($_this))
        {
            echo '<pre>',
            print_r($_this, TRUE),'</pre>';
            echo '<br />';
        }
    }
}