<?php if (!defined('BASEPATH')) exit('No direct script access allowed');
/**
* Modular Extensions - HMVC
* 
* Adapted from the CodeIgniter Core Classes
* @copyright	Copyright (c) 2006, EllisLab, Inc.
* @license		http://codeigniter.com/user_guide/license.html
* 
* Description:
* This library is used by the modules_helper to instantiate modules 
* which can operate as HMVC controllers.
* 
* Modules have access to the currently loaded core classes in the same
* manner as a normal controller.
* 
* Install this file as application/libraries/modular_extensions.php
*
* @version: 4.0.26 (c) Wiredesignz 2008-03-25
**/

class Module
{
	/** Constructor **/
	function Module()
    {
		$this->load =& new Loader($this);
		
		$ci =& get_instance();
		
		if (isset($ci->_me))
		{			
			$this->load->_home = $ci->_me['home'];
			
			unset($ci->_me);
			
			$this->_assign_libraries($ci);
			
			log_message('debug', get_class($this)." module initialised");
		}
		else
		{			
			log_message('debug', get_class($this)." method initialised");
		}
    }
	
	/** Module index method
	 * 
	 * Every module should have an index method.
	 * Generate a fatal error if it does not and index() is called.
	 **/
	function index()
    {
		show_error("Unable to locate the module index method: ".get_class($this).'/index');
    }

	/** Module error method
	 * 
	 * Called by modules::run if the requested $method does not exist.
	 * Loads and runs the $method sub-module if it can be found or
	 * Generate a fatal error if it is not found.
	 * 
	 * @param $method     string
	 * @param $data	      mixed   [optional]
	 * @param $show_error boolean [optional]
	 *  
	 * @return the method output
	 **/
    function error($method, $data = '', $show_error = TRUE)
    {		
		if ($this->load->method($method)) 
		{
			if (method_exists($this->$method, 'index'))
			{
				return $this->$method->index($data);
			}
		}

		if ($show_error)
		{
			show_error("Unable to run the requested method: ".get_class($this).'/'.$method);
		}
    }
	
	/** Assign libraries to a module **/
	function _assign_libraries(&$_parent)
	{		
		foreach (array_keys(get_object_vars($_parent)) as $key)
        {
            if (!isset($this->$key) AND $key != get_class($this)) 
            {
				$this->$key =& $_parent->$key;
            }
        }
	}
}

class Loader extends CI_Loader
{	
	/* module home directory */
	var $_home;
	
	/* the module */
	var $_me;

	/** Constructor **/
	function Loader(&$module)
	{
		parent::CI_Loader();

		$this->_me =& $module;
	}

    /** Assign module libraries **/
	function _assign_libraries_to(&$_sibling)
	{
		foreach (array_keys(get_object_vars($this->_me)) as $key)
        {
            if (!isset($_sibling->$key) AND $key != get_class($_sibling)) 
            {
				$_sibling->$key =& $this->_me->$key;
            }
        }
    }
	
	/** Upadte ci loaded classes **/
	function _update_ci_classes()
	{
		$ci =& get_instance();
		
		$ci->load->_ci_classes = array_merge($ci->load->_ci_classes, $this->_ci_classes);
		
		$this->_me->_assign_libraries($ci);
	}

	/** Load a module config file
	 * 
	 * @param $file         string  [optional]
	 * @param $use_sections boolean [optional]
	 **/
	function config($file = '', $use_sections = FALSE)
	{			
		$file = ($file == '') ? 'config' : strtolower($file);
		
		list($path, $file) = modules::path_to($file, $this->_home, 'config/');
		
		$current_config =& $this->_me->config->config;
		
		if (in_array($file, $this->_me->config->is_loaded, TRUE)) 
		{
			return;
		}
		
		if ($config = modules::load_file($file, $path, 'config'))
		{			
			if ($use_sections == TRUE)
			{
				if (isset($current_config[$file]))
				{
					$current_config[$file] = array_merge($current_config[$file], $config);
				}
				else
					$current_config[$file] = $config;
			}
			else
				$current_config = array_merge($current_config, $config);
			
			$this->_me->config->is_loaded[] = $file;
			
			unset($config);
		}
	}
	
	/** Load a module helper **/
	function helper($helper)
	{
		list($path, $helper) = modules::path_to($helper.'_helper', $this->_home, 'helpers/');
		
		$helper = strtolower($helper);
		
		if (isset($this->_ci_helpers[$helper]))
		{
			return;
		}
		
		if (is_file(BASEPATH.'helpers/'.$helper.EXT)) 
		{
			parent::helper($helper);
		}
				
		if ($path === FALSE) 
		{
			return;
		}
		
		modules::load_file($helper, $path);
		
		$this->_ci_helpers[$helper] = TRUE;				
	}
	
	/** Load a module language file **/	
	function language($langfile = array(), $lang = '')
	{		
		$deft_lang = $this->_me->config->item('language');
		
		$idiom = ($deft_lang == '') ? 'english' : $deft_lang;
		
		list($path, $langfile) = modules::path_to($langfile.'_lang', $this->_home, 'language/'.$idiom.'/');
		
		$langfile = strtolower($langfile);
		
		$current_language =& $this->_me->lang->language;
		
		if (in_array($langfile, $this->_me->lang->is_loaded, TRUE))
		{
			return;
		}
		
		if($lang = modules::load_file($langfile, $path, 'lang'))
		{
			$current_language = array_merge($current_language, $lang);
			
			$this->_me->lang->is_loaded[] = $langfile;
			
			unset($lang);
		}
	}
	
	/** Load a module library **/	
	function library($library, $params = NULL)
    {
		if ($library == 'database') return parent::database();
		
		list($path, $library) = modules::path_to($library, $this->_home, 'libraries/');
		
		$_library = strtolower($library);
		
		if (isset($this->_me->$_library))
        {
		   return;
     	}
		
		$library = ucfirst($library);
		
		if (is_file(BASEPATH.'libraries/'.$library.EXT) OR is_file(APPPATH.'libraries/'.$library.EXT)) 
		{
			parent::library($_library);
			
			$this->_update_ci_classes();
		}
		
		if ($path == FALSE)
		{
			return;
		}
		
		modules::load_file($_library, $path);
		
		$this->_me->$_library =& new $library($params);	
    }

	/** Load a module method **/
	function method($method)
	{
		list($path, $method) = modules::path_to($method, $this->_home, 'methods/');
		
		if(isset($this->_me->$method))
		{
			return TRUE;
		}
		
		if ($path == FALSE)
		{
			return FALSE;
		}
		
		modules::load_file($method, $path);
		
		$this->_me->$method =& new $method();
		
		unset($this->_me->$method->load);
		
		$this->_assign_libraries_to($this->_me->$method);
		
		return TRUE;
	}
	
	/** Load a module model **/	
	function model($model, $alias = FALSE, $connect = TRUE)
	{
		list($path, $model) = modules::path_to($model, $this->_home, 'models/');
		
		$_alias = ($alias == FALSE) ? strtolower($model) : strtolower($alias);

		if (isset($this->_me->$_alias))
		{
			return;
		}
		
		if(!class_exists('Model'))
		{
			load_class('Model', FALSE);
		}
		
		if ($connect) parent::database();
		
		modules::load_file($model, $path);
		
		$this->_me->$_alias =& new $model();
		
		$this->_assign_libraries_to($this->_me->$_alias);
	}
	
	/** Load a module plugin **/	
	function plugin($plugin)
	{
		list($path, $plugin) = modules::path_to($plugin.'_pi', $this->_home, 'plugins/');
		
		$plugin = strtolower($plugin);
		
		if (isset($this->_ci_plugins[$plugin]))
		{
			return;
		}
		
		if (is_file(BASEPATH.'plugins/'.$plugin.EXT)) 
		{
			parent::plugin($plugin);
		}
		
		if ($path === FALSE) 
		{
			return;
		}
		
		modules::load_file($plugin.'_pi', $path);
		
		$this->_ci_plugins[$plugin] = TRUE;
	}

	/** Load a module view **/
	function view($view, $data = array(), $return = TRUE)
	{
		list($path, $view) = modules::path_to($view, $this->_home, 'views/');
		
		$this->_ci_view_path = $path;
		
		return parent::view($view, $data, $return);
	}
}